@@ -1,11 +1,10 @@
 from pypy.interpreter.mixedmodule import MixedModule
-from pypy.module._hashlib.interp_hashlib import algorithms
+from pypy.module._hashlib.interp_hashlib import algorithms, fetch_names
 
 
 class Module(MixedModule):
     interpleveldefs = {
         'new' : 'interp_hashlib.new',
-        'openssl_md_meth_names': 'interp_hashlib.get(space).w_meth_names'
         }
 
     appleveldefs = {
@@ -15,5 +14,5 @@ class Module(MixedModule):
         interpleveldefs['openssl_' + name] = 'interp_hashlib.new_' + name
 
     def startup(self, space):
-        from rpython.rlib.ropenssl import init_digests
-        init_digests()
+        w_meth_names = fetch_names(space)
+        space.setattr(self, space.wrap('openssl_md_meth_names'), w_meth_names)
