@@ -3,6 +3,8 @@ from pypy.conftest import option
 
 
 class AppTestObjectDtypes(BaseNumpyAppTest):
+    spaceconfig = dict(usemodules=["micronumpy", "struct", "binascii"])
+
     def setup_class(cls):
         BaseNumpyAppTest.setup_class.im_func(cls)
         cls.w_runappdirect = cls.space.wrap(option.runappdirect)
@@ -170,7 +172,14 @@ class AppTestObjectDtypes(BaseNumpyAppTest):
         a = np.array([b'a' * 100], dtype='O')
         assert 'a' * 100 in str(a)
         b = a.astype('S')
+        assert b.dtype == 'S100'
         assert 'a' * 100 in str(b)
+        a = np.array([u'a' * 100], dtype='O')
+        assert 'a' * 100 in str(a)
+        b = a.astype('U')
+        assert b.dtype == 'U100'
+        assert 'a' * 100 in str(b)
+
         a = np.array([123], dtype='U')
         assert a[0] == u'123'
         b = a.astype('O')
@@ -187,3 +196,21 @@ class AppTestObjectDtypes(BaseNumpyAppTest):
         assert b.shape == (1,)
         assert b.dtype == np.float_
         assert (b == 1.0).all()
+
+
+    def test__reduce__(self):
+        from numpy import arange, dtype
+        from cPickle import loads, dumps
+        import sys
+        
+        a = arange(15).astype(object)
+        if '__pypy__' in sys.builtin_module_names:
+            raises(NotImplementedError, dumps, a)
+            skip('not implemented yet')
+        b = loads(dumps(a))
+        assert (a == b).all()
+
+        a = arange(15).astype(object).reshape((3, 5))
+        b = loads(dumps(a))
+        assert (a == b).all()
+        
