@@ -43,9 +43,9 @@ class TestString(BaseTestPyPyC):
             i25 = unicodegetitem(p13, i19)
             p27 = newstr(1)
             strsetitem(p27, 0, i23)
-            p30 = call(ConstClass(ll_str2unicode__rpy_stringPtr), p27, descr=...)
+            p30 = call_r(ConstClass(ll_str2unicode__rpy_stringPtr), p27, descr=...)
             guard_no_exception(descr=...)
-            i32 = call(ConstClass(_ll_2_str_eq_checknull_char__rpy_unicodePtr_UniChar), p30, i25, descr=...)
+            i32 = call_i(ConstClass(_ll_2_str_eq_checknull_char__rpy_unicodePtr_UniChar), p30, i25, descr=...)
             guard_true(i32, descr=...)
             i34 = int_add(i6, 1)
             --TICK--
@@ -80,12 +80,12 @@ class TestString(BaseTestPyPyC):
             i23 = strgetitem(p10, i19)
             p25 = newstr(1)
             strsetitem(p25, 0, i23)
-            p93 = call(ConstClass(fromstr), p25, 16, descr=<Callr . ri EF=4>)
+            p93 = call_r(ConstClass(fromstr), p25, 16, descr=<Callr . ri EF=4>)
             guard_no_exception(descr=...)
-            i95 = getfield_gc_pure(p93, descr=<FieldS rpython.rlib.rbigint.rbigint.inst_size .*>)
+            i95 = getfield_gc_pure_i(p93, descr=<FieldS rpython.rlib.rbigint.rbigint.inst_size .*>)
             i96 = int_gt(i95, #)
             guard_false(i96, descr=...)
-            i94 = call(ConstClass(rbigint._toint_helper), p93, descr=<Calli . r EF=4>)
+            i94 = call_i(ConstClass(rbigint._toint_helper), p93, descr=<Calli . r EF=4>)
             guard_no_exception(descr=...)
             i95 = int_add_ovf(i6, i94)
             guard_no_overflow(descr=...)
@@ -108,7 +108,7 @@ class TestString(BaseTestPyPyC):
             i79 = int_gt(i74, 0)
             guard_true(i79, descr=...)
             guard_not_invalidated(descr=...)
-            p80 = call(ConstClass(ll_int2dec__Signed), i74, descr=<Callr . i EF=3>)
+            p80 = call_r(ConstClass(ll_int2dec__Signed), i74, descr=<Callr . i EF=3>)
             guard_no_exception(descr=...)
             i85 = strlen(p80)
             p86 = new(descr=<SizeDescr .+>)
@@ -119,21 +119,21 @@ class TestString(BaseTestPyPyC):
             setfield_gc(p86, 23, descr=<FieldS stringbuilder.current_end .+>)
             setfield_gc(p86, 23, descr=<FieldS stringbuilder.total_size .+>)
             }}}
-            call(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
+            call_n(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
             guard_no_exception(descr=...)
-            i89 = getfield_gc(p86, descr=<FieldS stringbuilder.current_pos .+>)
-            i90 = getfield_gc(p86, descr=<FieldS stringbuilder.current_end .+>)
+            i89 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_pos .+>)
+            i90 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_end .+>)
             i91 = int_eq(i89, i90)
             cond_call(i91, ConstClass(ll_grow_by__stringbuilderPtr_Signed), p86, 1, descr=<Callv 0 ri EF=5>)
             guard_no_exception(descr=...)
-            i92 = getfield_gc(p86, descr=<FieldS stringbuilder.current_pos .+>)
+            i92 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_pos .+>)
             i93 = int_add(i92, 1)
-            p94 = getfield_gc(p86, descr=<FieldP stringbuilder.current_buf .+>)
+            p94 = getfield_gc_r(p86, descr=<FieldP stringbuilder.current_buf .+>)
             strsetitem(p94, i92, 32)
             setfield_gc(p86, i93, descr=<FieldS stringbuilder.current_pos .+>)
-            call(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
+            call_n(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
             guard_no_exception(descr=...)
-            p95 = call(..., descr=<Callr . r EF=5>)     # ll_build
+            p95 = call_r(..., descr=<Callr . r EF=5>)     # ll_build
             guard_no_exception(descr=...)
             i96 = strlen(p95)
             i97 = int_add_ovf(i71, i96)
@@ -176,7 +176,7 @@ class TestString(BaseTestPyPyC):
             strsetitem(p35, 3, 104)
             strsetitem(p35, 4, 95)
             copystrcontent(p31, p35, 0, 5, i32)
-            i49 = call(ConstClass(_ll_2_str_eq_nonnull__rpy_stringPtr_rpy_stringPtr), p35, ConstPtr(ptr48), descr=<Calli [48] rr EF=0 OS=28>)
+            i49 = call_i(ConstClass(_ll_2_str_eq_nonnull__rpy_stringPtr_rpy_stringPtr), p35, ConstPtr(ptr48), descr=<Calli [48] rr EF=0 OS=28>)
             guard_value(i49, 1, descr=...)
             ''')
 
@@ -195,7 +195,7 @@ class TestString(BaseTestPyPyC):
         loops = log.loops_by_filename(self.filepath)
         loop, = loops
         assert loop.match_by_id('callone', '''
-            p114 = call(ConstClass(ll_lower__rpy_stringPtr), p113, descr=<Callr . r EF=3>)
+            p114 = call_r(ConstClass(ll_lower__rpy_stringPtr), p113, descr=<Callr . r EF=3>)
             guard_no_exception(descr=...)
             ''')
         assert loop.match_by_id('calltwo', '')    # nothing
@@ -248,9 +248,9 @@ class TestString(BaseTestPyPyC):
         i50 = int_add(i47, 1)
         setfield_gc(p15, i50, descr=<FieldS pypy.module.__builtin__.functional.W_XRangeIterator.inst_current 8>)
         guard_not_invalidated(descr=...)
-        p80 = call(ConstClass(ll_str__IntegerR_SignedConst_Signed), i47, descr=<Callr . i EF=3>)
+        p80 = call_r(ConstClass(ll_str__IntegerR_SignedConst_Signed), i47, descr=<Callr . i EF=3>)
         guard_no_exception(descr=...)
-        p53 = call(ConstClass(fast_str_decode_ascii), p80, descr=<Callr . r EF=4>)
+        p53 = call_r(ConstClass(fast_str_decode_ascii), p80, descr=<Callr . r EF=4>)
         guard_no_exception(descr=...)
         guard_nonnull(p53, descr=...)
         --TICK--
