@@ -39,8 +39,6 @@ class FakeDescr(AbstractDescr):
         self.oopspecindex = oopspecindex
     def __repr__(self):
         return '<Descr>'
-    def as_vtable_size_descr(self):
-        return self
 
 class FakeDict(object):
     def __getitem__(self, key):
@@ -60,7 +58,7 @@ class FakeCPU:
         return FakeDescr()
     def fielddescrof(self, STRUCT, name):
         return FakeDescr()
-    def sizeof(self, STRUCT):
+    def sizeof(self, STRUCT, vtable=None):
         return FakeDescr()
     def arraydescrof(self, ARRAY):
         return FakeDescr()
@@ -142,6 +140,7 @@ class TestFlatten:
 
     def encoding_test(self, func, args, expected,
                       transform=False, liveness=False, cc=None, jd=None):
+        
         graphs = self.make_graphs(func, args)
         #graphs[0].show()
         if transform:
@@ -149,7 +148,8 @@ class TestFlatten:
             cc = cc or FakeCallControl()
             transform_graph(graphs[0], FakeCPU(self.rtyper), cc, jd)
         ssarepr = flatten_graph(graphs[0], fake_regallocs(),
-                                _include_all_exc_links=not transform)
+                                _include_all_exc_links=not transform,
+                                cpu=FakeCPU(self.rtyper))
         if liveness:
             from rpython.jit.codewriter.liveness import compute_liveness
             compute_liveness(ssarepr)
@@ -171,8 +171,8 @@ class TestFlatten:
             return n + 1
         self.encoding_test(f, [10], """
             int_gt %i0, $0 -> %i1
+            -live-
             goto_if_not %i1, L1
-            -live- L1
             int_copy %i0 -> %i2
             int_sub %i2, $3 -> %i3
             int_copy %i3 -> %i4
@@ -196,8 +196,8 @@ class TestFlatten:
             int_copy %i1 -> %i3
             L1:
             int_gt %i2, $0 -> %i4
+            -live-
             goto_if_not %i4, L2
-            -live- L2
             int_copy %i2 -> %i5
             int_copy %i3 -> %i6
             int_add %i6, %i5 -> %i7
@@ -220,8 +220,8 @@ class TestFlatten:
             int_copy %i0 -> %i2
             int_copy %i1 -> %i3
             L1:
+            -live-
             goto_if_not_int_gt %i2, $0, L2
-            -live- L2
             int_copy %i2 -> %i4
             int_copy %i3 -> %i5
             int_add %i5, %i4 -> %i6
@@ -459,8 +459,8 @@ class TestFlatten:
         # note that 'goto_if_not_int_is_true' is not the same thing
         # as just 'goto_if_not', because the last one expects a boolean
         self.encoding_test(f, [7], """
+            -live-
             goto_if_not_int_is_true %i0, L1
-            -live- L1
             int_return $False
             ---
             L1:
@@ -525,8 +525,8 @@ class TestFlatten:
             else:
                 return m2
         self.encoding_test(f, [4, 5, 6], """
+            -live- %i0, %i1, %i2
             goto_if_not_int_is_true %i0, L1
-            -live- %i1, %i2, L1
             int_return %i1
             ---
             L1:
@@ -540,15 +540,59 @@ class TestFlatten:
             except OverflowError:
                 return 42
         self.encoding_test(f, [7, 2], """
-            int_add_ovf %i0, %i1 -> %i2
-            -live- %i2
-            catch_exception L1
+            -live- %i0, %i1
+            int_add_jump_if_ovf L1, %i0, %i1 -> %i2
             int_return %i2
             ---
             L1:
             int_return $42
         """, transform=True, liveness=True)
 
+    def test_multiple_int_add_ovf(self):
+        def f(i, j):
+            try:
+                ovfcheck(j + i)
+                return ovfcheck(i + j)
+            except OverflowError:
+                return 42
+        self.encoding_test(f, [7, 2], """
+            -live- %i0, %i1
+            int_add_jump_if_ovf L1, %i1, %i0 -> %i2
+            int_copy %i1 -> %i3
+            int_copy %i0 -> %i4
+            -live- %i3, %i4
+            int_add_jump_if_ovf L2, %i4, %i3 -> %i5
+            int_return %i5
+            ---
+            L2:
+            int_return $42
+            ---
+            L1:
+            int_return $42
+        """, transform=True, liveness=True)
+
+    def test_ovfcheck_no_catch(self):
+        def f(i, j):
+            return ovfcheck(i + j)
+        err = py.test.raises(Exception, "self.encoding_test(f, [7, 2], '',"
+                             "transform=True, liveness=True)")
+        assert "ovfcheck()" in str(err.value)
+
+    def test_ovfcheck_reraise(self):
+        def f(i, j):
+            try:
+                ovfcheck(j + i)
+            except OverflowError:
+                raise
+        self.encoding_test(f, [7, 2], """
+            -live- %i0, %i1
+            int_add_jump_if_ovf L1, %i1, %i0 -> %i2
+            void_return
+            ---
+            L1:
+            raise $<* struct object>
+        """, transform=True, liveness=True)
+
     def test_residual_call_raising(self):
         @dont_look_inside
         def g(i, j):
