@@ -28,7 +28,7 @@ class RecursiveTests:
                 return 1
         res = self.meta_interp(main, [20], enable_opts='')
         assert res == main(20)
-        self.check_history(call=0)
+        self.check_history(call_i=0)
 
     def test_simple_recursion_with_exc(self):
         myjitdriver = JitDriver(greens=[], reds=['n', 'm'])
@@ -144,10 +144,10 @@ class RecursiveTests:
         f = self.get_interpreter(codes)
 
         assert self.meta_interp(f, [0, 0, 0], enable_opts='') == 42
-        self.check_resops(call_may_force=1, int_add=1, call=0)
+        self.check_resops(call_may_force_i=1, int_add=1, call=0)
         assert self.meta_interp(f, [0, 0, 0], enable_opts='',
                                 inline=True) == 42
-        self.check_resops(call=0, int_add=2, call_may_force=0,
+        self.check_resops(call=0, int_add=2, call_may_force_i=0,
                           guard_no_exception=0)
 
     def test_inline_jitdriver_check(self):
@@ -490,7 +490,7 @@ class RecursiveTests:
             return loop(100)
 
         res = self.meta_interp(main, [0], enable_opts='', trace_limit=TRACE_LIMIT)
-        self.check_resops(call=0, call_may_force=1)
+        self.check_resops(call=0, call_may_force_i=1)
 
         res = self.meta_interp(main, [1], enable_opts='', trace_limit=TRACE_LIMIT)
         self.check_resops(call=0, call_may_force=0)
@@ -575,7 +575,7 @@ class RecursiveTests:
                 result += f('-c-----------l-', i+100)
         self.meta_interp(g, [10], backendopt=True)
         self.check_aborted_count(1)
-        self.check_resops(call=0, call_assembler=2)
+        self.check_resops(call=0, call_assembler_i=2)
         self.check_jitcell_token_count(2)
 
     def test_directly_call_assembler(self):
@@ -592,7 +592,7 @@ class RecursiveTests:
                 i += 1
 
         self.meta_interp(portal, [2], inline=True)
-        self.check_history(call_assembler=1)
+        self.check_history(call_assembler_n=1)
 
     def test_recursion_cant_call_assembler_directly(self):
         driver = JitDriver(greens = ['codeno'], reds = ['i', 'j'],
@@ -624,7 +624,7 @@ class RecursiveTests:
         try:
             compile.compile_tmp_callback = my_ctc
             self.meta_interp(portal, [2, 5], inline=True)
-            self.check_resops(call_may_force=0, call_assembler=2)
+            self.check_resops(call_may_force=0, call_assembler_n=2)
         finally:
             compile.compile_tmp_callback = original_ctc
         # check that we made a temporary callback
@@ -679,7 +679,7 @@ class RecursiveTests:
         try:
             compile.compile_tmp_callback = my_ctc
             self.meta_interp(main, [2, 5], inline=True)
-            self.check_resops(call_may_force=0, call_assembler=2)
+            self.check_resops(call_may_force=0, call_assembler_n=2)
         finally:
             compile.compile_tmp_callback = original_ctc
         # check that we made a temporary callback
@@ -709,7 +709,7 @@ class RecursiveTests:
             return k
 
         self.meta_interp(portal, [2], inline=True)
-        self.check_history(call_assembler=1)
+        self.check_history(call_assembler_i=1)
 
     def test_directly_call_assembler_raise(self):
 
@@ -735,7 +735,7 @@ class RecursiveTests:
                 raise MyException(1)
 
         self.meta_interp(portal, [2], inline=True)
-        self.check_history(call_assembler=1)
+        self.check_history(call_assembler_n=1)
 
     def test_directly_call_assembler_fail_guard(self):
         driver = JitDriver(greens = ['codeno'], reds = ['i', 'k'],
@@ -1023,7 +1023,7 @@ class RecursiveTests:
         res = self.meta_interp(portal, [2, 0], inline=True,
                                policy=StopAtXPolicy(residual))
         assert res == portal(2, 0)
-        self.check_resops(call_assembler=4)
+        self.check_resops(call_assembler_i=4)
 
     def test_inline_without_hitting_the_loop(self):
         driver = JitDriver(greens = ['codeno'], reds = ['i'],
@@ -1073,7 +1073,7 @@ class RecursiveTests:
         assert portal(0, 1) == 2095
         res = self.meta_interp(portal, [0, 1], inline=True)
         assert res == 2095
-        self.check_resops(call_assembler=12)
+        self.check_resops(call_assembler_i=12)
 
     def test_inline_with_hitting_the_loop_sometimes_exc(self):
         driver = JitDriver(greens = ['codeno'], reds = ['i', 'k'],
@@ -1111,7 +1111,7 @@ class RecursiveTests:
         assert main(0, 1) == 2095
         res = self.meta_interp(main, [0, 1], inline=True)
         assert res == 2095
-        self.check_resops(call_assembler=12)
+        self.check_resops(call_assembler_n=12)
 
     def test_inline_recursion_limit(self):
         driver = JitDriver(greens = ["threshold", "loop"], reds=["i"])
@@ -1137,7 +1137,7 @@ class RecursiveTests:
         res1 = portal(10, True, 0)
         res2 = self.meta_interp(portal, [10, True, 0], inline=True)
         assert res1 == res2
-        self.check_resops(call_assembler=2)
+        self.check_resops(call_assembler_i=2)
 
         res1 = portal(9, True, 0)
         res2 = self.meta_interp(portal, [9, True, 0], inline=True)
