@@ -2,15 +2,25 @@ from __future__ import with_statement
 import py
 import sys
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
-from rpython.jit.metainterp.optimizeopt.optimizer import OptValue
-from rpython.jit.metainterp.optimizeopt.virtualize import VirtualValue, VArrayValue
-from rpython.jit.metainterp.optimizeopt.virtualize import VStructValue, AbstractVirtualValue
-from rpython.jit.metainterp.resume import *
-from rpython.jit.metainterp.history import BoxInt, BoxPtr, ConstInt
+from rpython.jit.metainterp.resume import ResumeDataVirtualAdder,\
+     AbstractResumeDataReader, get_VirtualCache_class, ResumeDataBoxReader,\
+     tag, TagOverflow, untag, tagged_eq, UNASSIGNED, TAGBOX, TAGVIRTUAL,\
+     tagged_list_eq, AbstractVirtualInfo, NUMBERING, TAGCONST, NULLREF,\
+     ResumeDataDirectReader, TAGINT, REF, VirtualInfo, VStructInfo,\
+     VArrayInfoNotClear, VStrPlainInfo, VStrConcatInfo, VStrSliceInfo,\
+     VUniPlainInfo, VUniConcatInfo, VUniSliceInfo, Snapshot, FrameInfo,\
+     capture_resumedata, ResumeDataLoopMemo, UNASSIGNEDVIRTUAL, INT,\
+     annlowlevel, PENDINGFIELDSP
+from rpython.jit.metainterp.optimizeopt import info
+from rpython.jit.metainterp.history import ConstInt, Const, AbstractDescr
 from rpython.jit.metainterp.history import ConstPtr, ConstFloat
 from rpython.jit.metainterp.optimizeopt.test.test_util import LLtypeMixin
 from rpython.jit.metainterp import executor
 from rpython.jit.codewriter import heaptracker, longlong
+from rpython.jit.metainterp.resoperation import ResOperation, InputArgInt,\
+     InputArgRef, rop
+from rpython.rlib.debug import debug_start, debug_stop, debug_print,\
+     have_debug_prints
 
 class Storage:
     rd_frame_info_list = None
@@ -22,15 +32,19 @@ class Storage:
 
 
 class FakeOptimizer(object):
-    def __init__(self, values):
-        self.values = values
-        
-    def getvalue(self, box):
-        try:
-            value = self.values[box]
-        except KeyError:
-            value = self.values[box] = OptValue(box)
-        return value
+    def get_box_replacement(self, op):
+        while (op.get_forwarded() is not None and
+               not isinstance(op.get_forwarded(), info.AbstractInfo)):
+            op = op.get_forwarded()
+        return op
+
+    def getrawptrinfo(self, op, create=True):
+        op = self.get_box_replacement(op)
+        return op.get_forwarded()
+
+    def getptrinfo(self, op, create=True):
+        op = self.get_box_replacement(op)
+        return op.get_forwarded()        
 
 
 # ____________________________________________________________
@@ -38,6 +52,7 @@ class FakeOptimizer(object):
 def dump_storage(storage, liveboxes):
     "For profiling only."
     debug_start("jit-resume")
+    return # XXX refactor if needed
     if have_debug_prints():
         debug_print('Log storage', compute_unique_id(storage))
         frameinfo = storage.rd_frame_info_list
@@ -120,11 +135,11 @@ def test_reuse_vinfo():
             self.fieldnums = fieldnums
         def equals(self, fieldnums):
             return self.fieldnums == fieldnums
-    class FakeVirtualValue(AbstractVirtualValue):
+    class FakeVirtualValue(info.AbstractVirtualPtrInfo):
         def visitor_dispatch_virtual_type(self, *args):
             return FakeVInfo()
-    modifier = ResumeDataVirtualAdder(None, None, None)
-    v1 = FakeVirtualValue(None, None)
+    modifier = ResumeDataVirtualAdder(None, None, None, None)
+    v1 = FakeVirtualValue()
     vinfo1 = modifier.make_virtual_info(v1, [1, 2, 4])
     vinfo2 = modifier.make_virtual_info(v1, [1, 2, 4])
     assert vinfo1 is vinfo2
@@ -133,6 +148,15 @@ def test_reuse_vinfo():
     vinfo4 = modifier.make_virtual_info(v1, [1, 2, 6])
     assert vinfo3 is vinfo4
 
+def setvalue(op, val):
+    if op.type == 'i':
+        op.setint(val)
+    elif op.type == 'r':
+        op.setref_base(val)
+    elif op.type == 'f':
+        op.setfloatstorage(val)
+    else:
+        assert op.type == 'v'
 
 class MyMetaInterp:
     _already_allocated_resume_virtuals = None
@@ -144,7 +168,6 @@ class MyMetaInterp:
         self.cpu = cpu
         self.trace = []
         self.framestack = []
-        self.resboxes = []
 
     def newframe(self, jitcode):
         frame = FakeFrame(jitcode, -1)
@@ -152,18 +175,14 @@ class MyMetaInterp:
         return frame    
 
     def execute_and_record(self, opnum, descr, *argboxes):
-        resbox = executor.execute(self.cpu, None, opnum, descr, *argboxes)
-        self.trace.append((opnum,
-                           list(argboxes),
-                           resbox,
-                           descr))
-        if resbox is not None:
-            self.resboxes.append(resbox)
-        return resbox
+        resvalue = executor.execute(self.cpu, None, opnum, descr, *argboxes)
+        op = ResOperation(opnum, list(argboxes), descr)
+        setvalue(op, resvalue)
+        self.trace.append((opnum, list(argboxes), resvalue, descr))
+        return op
 
-    def execute_new_with_vtable(self, known_class):
-        return self.execute_and_record(rop.NEW_WITH_VTABLE, None,
-                                       known_class)
+    def execute_new_with_vtable(self, descr=None):
+        return self.execute_and_record(rop.NEW_WITH_VTABLE, descr)
 
     def execute_new(self, typedescr):
         return self.execute_and_record(rop.NEW, typedescr)
@@ -263,7 +282,7 @@ def Numbering(prev, nums):
     return numb
 
 def test_simple_read():
-    #b1, b2, b3 = [BoxInt(), BoxPtr(), BoxInt()]
+    #b1, b2, b3 = [BoxInt(), InputArgRef(), BoxInt()]
     c1, c2, c3 = [ConstInt(111), ConstInt(222), ConstInt(333)]
     storage = Storage()
     storage.rd_consts = [c1, c2, c3]
@@ -350,8 +369,8 @@ def test_prepare_virtuals():
 class FakeResumeDataReader(AbstractResumeDataReader):
     VirtualCache = get_VirtualCache_class('Fake')
     
-    def allocate_with_vtable(self, known_class):
-        return FakeBuiltObject(vtable=known_class)
+    def allocate_with_vtable(self, descr):
+        return FakeBuiltObject(vtable=descr)
     def allocate_struct(self, typedescr):
         return FakeBuiltObject(typedescr=typedescr)
     def allocate_array(self, length, arraydescr, clear):
@@ -537,7 +556,7 @@ def test_Numbering_create():
     assert list(numb1.nums) == l1
 
 def test_capture_resumedata():
-    b1, b2, b3 = [BoxInt(), BoxPtr(), BoxInt()]
+    b1, b2, b3 = [InputArgInt(), InputArgRef(), InputArgInt()]
     c1, c2, c3 = [ConstInt(1), ConstInt(2), ConstInt(3)]
     fs = [FakeFrame("code0", 0, b1, c1, b2)]
 
@@ -610,7 +629,7 @@ class FakeMetaInterpStaticData:
 
 def test_rebuild_from_resumedata():
     py.test.skip("XXX rewrite")
-    b1, b2, b3 = [BoxInt(), BoxPtr(), BoxInt()]
+    b1, b2, b3 = [BoxInt(), InputArgRef(), BoxInt()]
     c1, c2, c3 = [ConstInt(1), ConstInt(2), ConstInt(3)]    
     storage = Storage()
     fs = [FakeFrame("code0", 0, b1, c1, b2),
@@ -618,11 +637,11 @@ def test_rebuild_from_resumedata():
           FakeFrame("code2", 9, c3, b2)]
     capture_resumedata(fs, None, [], storage)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
-    liveboxes = modifier.finish(FakeOptimizer({}))
+    modifier = ResumeDataVirtualAdder(None, storage, storage, memo)
+    liveboxes = modifier.finish(FakeOptimizer())
     metainterp = MyMetaInterp()
 
-    b1t, b2t, b3t = [BoxInt(), BoxPtr(), BoxInt()]
+    b1t, b2t, b3t = [BoxInt(), InputArgRef(), BoxInt()]
     newboxes = _resume_remap(liveboxes, [b1, b2, b3], b1t, b2t, b3t)
 
     result = rebuild_from_resumedata(metainterp, storage, False)
@@ -634,7 +653,7 @@ def test_rebuild_from_resumedata():
 
 def test_rebuild_from_resumedata_with_virtualizable():
     py.test.skip("XXX rewrite")
-    b1, b2, b3, b4 = [BoxInt(), BoxPtr(), BoxInt(), BoxPtr()]
+    b1, b2, b3, b4 = [BoxInt(), InputArgRef(), BoxInt(), InputArgRef()]
     c1, c2, c3 = [ConstInt(1), ConstInt(2), ConstInt(3)]    
     storage = Storage()
     fs = [FakeFrame("code0", 0, b1, c1, b2),
@@ -642,11 +661,11 @@ def test_rebuild_from_resumedata_with_virtualizable():
           FakeFrame("code2", 9, c3, b2)]
     capture_resumedata(fs, [b4], [], storage)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, memo)
+    modifier = ResumeDataVirtualAdder(None, storage, memo)
     liveboxes = modifier.finish(FakeOptimizer({}))
     metainterp = MyMetaInterp()
 
-    b1t, b2t, b3t, b4t = [BoxInt(), BoxPtr(), BoxInt(), BoxPtr()]
+    b1t, b2t, b3t, b4t = [BoxInt(), InputArgRef(), BoxInt(), InputArgRef()]
     newboxes = _resume_remap(liveboxes, [b1, b2, b3, b4], b1t, b2t, b3t, b4t)
 
     result = rebuild_from_resumedata(metainterp, newboxes, storage,
@@ -659,7 +678,7 @@ def test_rebuild_from_resumedata_with_virtualizable():
 
 def test_rebuild_from_resumedata_two_guards():
     py.test.skip("XXX rewrite")
-    b1, b2, b3, b4 = [BoxInt(), BoxPtr(), BoxInt(), BoxInt()]
+    b1, b2, b3, b4 = [BoxInt(), InputArgRef(), BoxInt(), BoxInt()]
     c1, c2, c3 = [ConstInt(1), ConstInt(2), ConstInt(3)]    
     storage = Storage()
     fs = [FakeFrame("code0", 0, b1, c1, b2),
@@ -671,15 +690,15 @@ def test_rebuild_from_resumedata_two_guards():
     capture_resumedata(fs, None, [], storage2)
     
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, memo)
+    modifier = ResumeDataVirtualAdder(None, storage, memo)
     liveboxes = modifier.finish(FakeOptimizer({}))
 
-    modifier = ResumeDataVirtualAdder(storage2, memo)
+    modifier = ResumeDataVirtualAdder(None, storage2, memo)
     liveboxes2 = modifier.finish(FakeOptimizer({}))
 
     metainterp = MyMetaInterp()
 
-    b1t, b2t, b3t, b4t = [BoxInt(), BoxPtr(), BoxInt(), BoxInt()]
+    b1t, b2t, b3t, b4t = [BoxInt(), InputArgRef(), BoxInt(), BoxInt()]
     newboxes = _resume_remap(liveboxes, [b1, b2, b3], b1t, b2t, b3t)
 
     result = rebuild_from_resumedata(metainterp, newboxes, storage,
@@ -721,7 +740,7 @@ def virtual_value(keybox, value, next):
 def test_rebuild_from_resumedata_two_guards_w_virtuals():
     py.test.skip("XXX rewrite")
     
-    b1, b2, b3, b4, b5 = [BoxInt(), BoxPtr(), BoxInt(), BoxInt(), BoxInt()]
+    b1, b2, b3, b4, b5 = [BoxInt(), InputArgRef(), BoxInt(), BoxInt(), BoxInt()]
     c1, c2, c3, c4 = [ConstInt(1), ConstInt(2), ConstInt(3),
                       LLtypeMixin.nodebox.constbox()]
     storage = Storage()
@@ -741,7 +760,7 @@ def test_rebuild_from_resumedata_two_guards_w_virtuals():
     assert storage.rd_virtuals[0].fieldnums == [tag(-1, TAGBOX),
                                                 tag(0, TAGCONST)]
 
-    b6 = BoxPtr()
+    b6 = InputArgRef()
     v6 = virtual_value(b6, c2, None)
     v6.setfield(LLtypeMixin.nextdescr, v6)    
     values = {b2: virtual_value(b2, b4, v6), b6: v6}
@@ -783,7 +802,7 @@ def test_rebuild_from_resumedata_two_guards_w_virtuals():
 
 def test_rebuild_from_resumedata_two_guards_w_shared_virtuals():
     py.test.skip("XXX rewrite")
-    b1, b2, b3, b4, b5, b6 = [BoxPtr(), BoxPtr(), BoxInt(), BoxPtr(), BoxInt(), BoxInt()]
+    b1, b2, b3, b4, b5, b6 = [InputArgRef(), InputArgRef(), BoxInt(), InputArgRef(), BoxInt(), BoxInt()]
     c1, c2, c3, c4 = [ConstInt(1), ConstInt(2), ConstInt(3),
                       LLtypeMixin.nodebox.constbox()]
     storage = Storage()
@@ -811,7 +830,7 @@ def test_rebuild_from_resumedata_two_guards_w_shared_virtuals():
 
 def test_resumedata_top_recursive_virtuals():
     py.test.skip("XXX rewrite")
-    b1, b2, b3 = [BoxPtr(), BoxPtr(), BoxInt()]
+    b1, b2, b3 = [InputArgRef(), InputArgRef(), BoxInt()]
     storage = Storage()
     fs = [FakeFrame("code0", 0, b1, b2)]
     capture_resumedata(fs, None, [], storage)
@@ -883,7 +902,8 @@ def test_ResumeDataLoopMemo_other():
     assert memo.consts[index] is const
 
 def test_ResumeDataLoopMemo_number():
-    b1, b2, b3, b4, b5 = [BoxInt(), BoxInt(), BoxInt(), BoxPtr(), BoxPtr()]
+    b1, b2, b3, b4, b5 = [InputArgInt(), InputArgInt(), InputArgInt(),
+                          InputArgRef(), InputArgRef()]
     c1, c2, c3, c4 = [ConstInt(1), ConstInt(2), ConstInt(3), ConstInt(4)]    
 
     env = [b1, c1, b2, b1, c2]
@@ -895,7 +915,7 @@ def test_ResumeDataLoopMemo_number():
 
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
 
-    numb, liveboxes, v = memo.number(FakeOptimizer({}), snap1)
+    numb, liveboxes, v = memo.number(FakeOptimizer(), snap1)
     assert v == 0
 
     assert liveboxes == {b1: tag(0, TAGBOX), b2: tag(1, TAGBOX),
@@ -907,7 +927,7 @@ def test_ResumeDataLoopMemo_number():
                                     tag(0, TAGBOX), tag(2, TAGINT)]
     assert not numb.prev.prev
 
-    numb2, liveboxes2, v = memo.number(FakeOptimizer({}), snap2)
+    numb2, liveboxes2, v = memo.number(FakeOptimizer(), snap2)
     assert v == 0
     
     assert liveboxes2 == {b1: tag(0, TAGBOX), b2: tag(1, TAGBOX),
@@ -920,20 +940,16 @@ def test_ResumeDataLoopMemo_number():
     env3 = [c3, b3, b1, c3]
     snap3 = Snapshot(snap, env3)
 
-    class FakeValue(object):
-        def __init__(self, virt, box):
+    class FakeVirtualInfo(info.AbstractInfo):
+        def __init__(self, virt):
             self.virt = virt
-            self.valuebox = box
-
-        def get_key_box(self):
-            return self.valuebox
 
         def is_virtual(self):
             return self.virt
 
     # renamed
-    numb3, liveboxes3, v = memo.number(FakeOptimizer({b3: FakeValue(False, c4)}),
-                                       snap3)
+    b3.set_forwarded(c4)
+    numb3, liveboxes3, v = memo.number(FakeOptimizer(), snap3)
     assert v == 0
     
     assert liveboxes3 == {b1: tag(0, TAGBOX), b2: tag(1, TAGBOX)}
@@ -943,10 +959,10 @@ def test_ResumeDataLoopMemo_number():
 
     # virtual
     env4 = [c3, b4, b1, c3]
-    snap4 = Snapshot(snap, env4)    
+    snap4 = Snapshot(snap, env4)
 
-    numb4, liveboxes4, v = memo.number(FakeOptimizer({b4: FakeValue(True, b4)}),
-                                       snap4)
+    b4.set_forwarded(FakeVirtualInfo(True))
+    numb4, liveboxes4, v = memo.number(FakeOptimizer(), snap4)
     assert v == 1
     
     assert liveboxes4 == {b1: tag(0, TAGBOX), b2: tag(1, TAGBOX),
@@ -956,11 +972,11 @@ def test_ResumeDataLoopMemo_number():
     assert numb4.prev == numb.prev
 
     env5 = [b1, b4, b5]
-    snap5 = Snapshot(snap4, env5)    
+    snap5 = Snapshot(snap4, env5)
 
-    numb5, liveboxes5, v = memo.number(FakeOptimizer({b4: FakeValue(True, b4),
-                                                      b5: FakeValue(True, b5)}),
-                                       snap5)
+    b4.set_forwarded(FakeVirtualInfo(True))
+    b5.set_forwarded(FakeVirtualInfo(True))
+    numb5, liveboxes5, v = memo.number(FakeOptimizer(), snap5)
     assert v == 2
     
     assert liveboxes5 == {b1: tag(0, TAGBOX), b2: tag(1, TAGBOX),
@@ -971,7 +987,7 @@ def test_ResumeDataLoopMemo_number():
 
 def test_ResumeDataLoopMemo_number_boxes():
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    b1, b2 = [BoxInt(), BoxInt()]
+    b1, b2 = [InputArgInt(), InputArgInt()]
     assert memo.num_cached_boxes() == 0
     boxes = []
     num = memo.assign_number_to_box(b1, boxes)
@@ -1000,7 +1016,7 @@ def test_ResumeDataLoopMemo_number_boxes():
 
 def test_ResumeDataLoopMemo_number_virtuals():
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    b1, b2 = [BoxInt(), BoxInt()]
+    b1, b2 = [InputArgInt(), InputArgInt()]
     assert memo.num_cached_virtuals() == 0
     num = memo.assign_number_to_virtual(b1)
     assert num == -1
@@ -1020,9 +1036,9 @@ def test_ResumeDataLoopMemo_number_virtuals():
     assert memo.num_cached_virtuals() == 0
 
 def test_register_virtual_fields():
-    b1, b2 = BoxInt(), BoxInt()
-    vbox = BoxPtr()
-    modifier = ResumeDataVirtualAdder(None, None, None)
+    b1, b2 = InputArgInt(), InputArgInt()
+    vbox = InputArgRef()
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), None, None, None)
     modifier.liveboxes_from_env = {}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
@@ -1031,7 +1047,7 @@ def test_register_virtual_fields():
                                   b2: UNASSIGNED}
     assert modifier.vfieldboxes == {vbox: [b1, b2]}
 
-    modifier = ResumeDataVirtualAdder(None, None, None)
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), None, None, None)
     modifier.liveboxes_from_env = {vbox: tag(0, TAGVIRTUAL)}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
@@ -1061,8 +1077,8 @@ def test_virtual_adder_int_constants():
     b1s, b2s, b3s = [ConstInt(sys.maxint), ConstInt(2**16), ConstInt(-65)]
     storage = make_storage(b1s, b2s, b3s)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())    
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
-    liveboxes = modifier.finish(FakeOptimizer({}))
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
+    liveboxes = modifier.finish(FakeOptimizer())
     assert storage.rd_snapshot is None
     cpu = MyCPU([])
     reader = ResumeDataDirectReader(MyMetaInterp(cpu), storage, "deadframe")
@@ -1075,15 +1091,16 @@ def test_virtual_adder_memo_const_sharing():
     b1s, b2s, b3s = [ConstInt(sys.maxint), ConstInt(2**16), ConstInt(-65)]
     storage = make_storage(b1s, b2s, b3s)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
-    modifier.finish(FakeOptimizer({}))
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
+    modifier.finish(FakeOptimizer())
     assert len(memo.consts) == 2
     assert storage.rd_consts is memo.consts
 
     b1s, b2s, b3s = [ConstInt(sys.maxint), ConstInt(2**17), ConstInt(-65)]
     storage2 = make_storage(b1s, b2s, b3s)
-    modifier2 = ResumeDataVirtualAdder(storage2, storage2, memo)
-    modifier2.finish(FakeOptimizer({}))
+    modifier2 = ResumeDataVirtualAdder(FakeOptimizer(), storage2, storage2,
+                                       memo)
+    modifier2.finish(FakeOptimizer())
     assert len(memo.consts) == 3    
     assert storage2.rd_consts is memo.consts
 
@@ -1134,24 +1151,17 @@ class ResumeDataFakeReader(ResumeDataBoxReader):
 
 
 def test_virtual_adder_no_op_renaming():
-    b1s, b2s, b3s = [BoxInt(1), BoxInt(2), BoxInt(3)]
+    b1s, b2s, b3s = [InputArgInt(1), InputArgInt(2), InputArgInt(3)]
     storage = make_storage(b1s, b2s, b3s)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
-    b1_2 = BoxInt()
-    class FakeValue(object):
+    b1_2 = InputArgInt()
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
 
-        def is_virtual(self):
-            return False
-
-        def get_key_box(self):
-            return b1_2
-
-    val = FakeValue()
-    values = {b1s: val, b2s: val}  
-    liveboxes = modifier.finish(FakeOptimizer(values))
+    b1s.set_forwarded(b1_2)
+    b2s.set_forwarded(b1_2)
+    liveboxes = modifier.finish(FakeOptimizer())
     assert storage.rd_snapshot is None
-    b1t, b3t = [BoxInt(11), BoxInt(33)]
+    b1t, b3t = [InputArgInt(11), InputArgInt(33)]
     newboxes = _resume_remap(liveboxes, [b1_2, b3s], b1t, b3t)
     metainterp = MyMetaInterp()
     reader = ResumeDataFakeReader(storage, newboxes, metainterp)
@@ -1165,13 +1175,13 @@ def test_virtual_adder_no_op_renaming():
 
 
 def test_virtual_adder_make_constant():
-    b1s, b2s, b3s = [BoxInt(1), BoxPtr(), BoxInt(3)]
+    b1s, b2s, b3s = [InputArgInt(1), InputArgRef(), InputArgInt(3)]
     b1s = ConstInt(111)
     storage = make_storage(b1s, b2s, b3s)
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())        
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
-    liveboxes = modifier.finish(FakeOptimizer({}))
-    b2t, b3t = [BoxPtr(demo55o), BoxInt(33)]
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
+    liveboxes = modifier.finish(FakeOptimizer())
+    b2t, b3t = [InputArgRef(demo55o), InputArgInt(33)]
     newboxes = _resume_remap(liveboxes, [b2s, b3s], b2t, b3t)
     metainterp = MyMetaInterp()
     reader = ResumeDataFakeReader(storage, newboxes, metainterp)
@@ -1186,38 +1196,38 @@ def test_virtual_adder_make_constant():
 
 
 def test_virtual_adder_make_virtual():
-    b2s, b3s, b4s, b5s = [BoxPtr(), BoxInt(3), BoxPtr(), BoxPtr()]  
+    b2s, b3s, b4s, b5s = [InputArgRef(), InputArgInt(3), InputArgRef(),
+                          InputArgRef()]  
     c1s = ConstInt(111)
     storage = Storage()
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
     modifier.liveboxes_from_env = {}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
 
-    v4 = VirtualValue(fakeoptimizer, ConstAddr(LLtypeMixin.node_vtable_adr2,
-                                                LLtypeMixin.cpu), b4s)
-    v4.setfield(LLtypeMixin.nextdescr, OptValue(b2s))
-    v4.setfield(LLtypeMixin.valuedescr, OptValue(b3s))
-    v4.setfield(LLtypeMixin.otherdescr, OptValue(b5s))
-    v4._cached_sorted_fields = [LLtypeMixin.nextdescr, LLtypeMixin.valuedescr,
-                                LLtypeMixin.otherdescr]
-    v2 = VirtualValue(fakeoptimizer, ConstAddr(LLtypeMixin.node_vtable_adr,
-                                                LLtypeMixin.cpu), b2s)
-    v2.setfield(LLtypeMixin.nextdescr, v4)
-    v2.setfield(LLtypeMixin.valuedescr, OptValue(c1s))
-    v2._cached_sorted_fields = [LLtypeMixin.nextdescr, LLtypeMixin.valuedescr]
+    vdescr = LLtypeMixin.nodesize2
+    ca = ConstAddr(LLtypeMixin.node_vtable_adr2, LLtypeMixin.cpu)
+    v4 = info.InstancePtrInfo(vdescr, ca, True)
+    b4s.set_forwarded(v4)
+    v4.setfield(LLtypeMixin.nextdescr, ca, b2s)
+    v4.setfield(LLtypeMixin.valuedescr, ca, b3s)
+    v4.setfield(LLtypeMixin.otherdescr, ca, b5s)
+    ca = ConstAddr(LLtypeMixin.node_vtable_adr, LLtypeMixin.cpu)
+    v2 = info.InstancePtrInfo(LLtypeMixin.nodesize, ca, True)
+    v2.setfield(LLtypeMixin.nextdescr, b4s, ca)
+    v2.setfield(LLtypeMixin.valuedescr, c1s, ca)
+    b2s.set_forwarded(v2)
 
-    modifier.register_virtual_fields(b2s, [b4s, c1s])
-    modifier.register_virtual_fields(b4s, [b2s, b3s, b5s])
-    values = {b2s: v2, b4s: v4}
+    modifier.register_virtual_fields(b2s, [c1s, None, None, None, b4s])
+    modifier.register_virtual_fields(b4s, [b3s, None, None, None, b2s, b5s])
 
     liveboxes = []
-    modifier._number_virtuals(liveboxes, FakeOptimizer(values), 0)
+    modifier._number_virtuals(liveboxes, FakeOptimizer(), 0)
     storage.rd_consts = memo.consts[:]
     storage.rd_numb = None
     # resume
-    b3t, b5t = [BoxInt(33), BoxPtr(demo55o)]
+    b3t, b5t = [InputArgInt(33), InputArgRef(demo55o)]
     newboxes = _resume_remap(liveboxes, [#b2s -- virtual
                                          b3s,
                                          #b4s -- virtual
@@ -1231,12 +1241,8 @@ def test_virtual_adder_make_virtual():
     b2t = reader.decode_ref(modifier._gettagged(b2s))
     b4t = reader.decode_ref(modifier._gettagged(b4s))
     trace = metainterp.trace
-    b2new = (rop.NEW_WITH_VTABLE, [ConstAddr(LLtypeMixin.node_vtable_adr,
-                                         LLtypeMixin.cpu)],
-                              b2t, None)
-    b4new = (rop.NEW_WITH_VTABLE, [ConstAddr(LLtypeMixin.node_vtable_adr2,
-                                         LLtypeMixin.cpu)],
-                              b4t, None)
+    b2new = (rop.NEW_WITH_VTABLE, [], b2t.getref_base(), LLtypeMixin.nodesize)
+    b4new = (rop.NEW_WITH_VTABLE, [], b4t.getref_base(), LLtypeMixin.nodesize2)
     b2set = [(rop.SETFIELD_GC, [b2t, b4t],      None, LLtypeMixin.nextdescr),
              (rop.SETFIELD_GC, [b2t, c1s],      None, LLtypeMixin.valuedescr)]
     b4set = [(rop.SETFIELD_GC, [b4t, b2t],     None, LLtypeMixin.nextdescr),
@@ -1247,12 +1253,13 @@ def test_virtual_adder_make_virtual():
     # check that we get the operations in 'expected', in a possibly different
     # order.
     assert len(trace) == len(expected)
+    orig = trace[:]
     with CompareableConsts():
         for x in trace:
             assert x in expected
             expected.remove(x)
 
-    ptr = b2t.value._obj.container._as_ptr()
+    ptr = b2t.getref_base()._obj.container._as_ptr()
     assert lltype.typeOf(ptr) == lltype.Ptr(LLtypeMixin.NODE)
     assert ptr.value == 111
     ptr2 = ptr.next
@@ -1269,26 +1276,26 @@ class CompareableConsts(object):
         del Const.__eq__
 
 def test_virtual_adder_make_varray():
-    b2s, b4s = [BoxPtr(), BoxInt(4)]
+    b2s, b4s = [InputArgRef(), InputArgInt(4)]
     c1s = ConstInt(111)
     storage = Storage()
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
     modifier.liveboxes_from_env = {}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
 
-    v2 = VArrayValue(LLtypeMixin.arraydescr, None, 2, b2s)
+    v2 = info.ArrayPtrInfo(LLtypeMixin.arraydescr, size=2, is_virtual=True)
+    b2s.set_forwarded(v2)
     v2._items = [b4s, c1s]
     modifier.register_virtual_fields(b2s, [b4s, c1s])
     liveboxes = []
-    values = {b2s: v2}
-    modifier._number_virtuals(liveboxes, FakeOptimizer(values), 0)
+    modifier._number_virtuals(liveboxes, FakeOptimizer(), 0)
     dump_storage(storage, liveboxes)
     storage.rd_consts = memo.consts[:]
     storage.rd_numb = None
     # resume
-    b1t, b3t, b4t = [BoxInt(11), BoxInt(33), BoxInt(44)]
+    b1t, b3t, b4t = [InputArgInt(11), InputArgInt(33), InputArgInt(44)]
     newboxes = _resume_remap(liveboxes, [#b2s -- virtual
                                          b4s],
                                          b4t)
@@ -1299,7 +1306,8 @@ def test_virtual_adder_make_varray():
     b2t = reader.decode_ref(tag(0, TAGVIRTUAL))
     trace = metainterp.trace
     expected = [
-        (rop.NEW_ARRAY, [ConstInt(2)], b2t, LLtypeMixin.arraydescr),
+        (rop.NEW_ARRAY, [ConstInt(2)], b2t.getref_base(),
+         LLtypeMixin.arraydescr),
         (rop.SETARRAYITEM_GC, [b2t,ConstInt(0), b4t],None,
                               LLtypeMixin.arraydescr),
         (rop.SETARRAYITEM_GC, [b2t,ConstInt(1), c1s], None,
@@ -1309,7 +1317,7 @@ def test_virtual_adder_make_varray():
         for x, y in zip(expected, trace):
             assert x == y
     #
-    ptr = b2t.value._obj.container._as_ptr()
+    ptr = b2t.getref_base()._obj.container._as_ptr()
     assert lltype.typeOf(ptr) == lltype.Ptr(lltype.GcArray(lltype.Signed))
     assert len(ptr) == 2
     assert ptr[0] == 44
@@ -1317,24 +1325,26 @@ def test_virtual_adder_make_varray():
 
 
 def test_virtual_adder_make_vstruct():
-    b2s, b4s = [BoxPtr(), BoxPtr()]
+    b2s, b4s = [InputArgRef(), InputArgRef()]
     c1s = ConstInt(111)
     storage = Storage()
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
+    modifier = ResumeDataVirtualAdder(FakeOptimizer(), storage, storage, memo)
     modifier.liveboxes_from_env = {}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
-    v2 = VStructValue(fakeoptimizer, LLtypeMixin.ssize, b2s)
-    v2.setfield(LLtypeMixin.adescr, OptValue(c1s))
-    v2.setfield(LLtypeMixin.bdescr, OptValue(b4s))
-    modifier.register_virtual_fields(b2s, [c1s, b4s])
+    v2 = info.StructPtrInfo(LLtypeMixin.ssize, is_virtual=True)
+    b2s.set_forwarded(v2)
+    v2.setfield(LLtypeMixin.adescr, b2s, c1s)
+    v2.setfield(LLtypeMixin.abisdescr, b2s, c1s)
+    v2.setfield(LLtypeMixin.bdescr, b2s, b4s)
+    modifier.register_virtual_fields(b2s, [c1s, c1s, b4s])
     liveboxes = []
-    modifier._number_virtuals(liveboxes, FakeOptimizer({b2s: v2}), 0)
+    modifier._number_virtuals(liveboxes, FakeOptimizer(), 0)
     dump_storage(storage, liveboxes)
     storage.rd_consts = memo.consts[:]
     storage.rd_numb = None
-    b4t = BoxPtr()
+    b4t = InputArgRef()
     newboxes = _resume_remap(liveboxes, [#b2s -- virtual
                                          b4s], b4t)
     #
@@ -1346,45 +1356,44 @@ def test_virtual_adder_make_vstruct():
 
     trace = metainterp.trace
     expected = [
-        (rop.NEW, [], b2t, LLtypeMixin.ssize),
+        (rop.NEW, [], b2t.getref_base(), LLtypeMixin.ssize),
         (rop.SETFIELD_GC, [b2t, c1s],  None, LLtypeMixin.adescr),
+        (rop.SETFIELD_GC, [b2t, c1s],  None, LLtypeMixin.abisdescr),
         (rop.SETFIELD_GC, [b2t, b4t], None, LLtypeMixin.bdescr),
         ]
     with CompareableConsts():
         for x, y in zip(expected, trace):
             assert x == y
     #
-    ptr = b2t.value._obj.container._as_ptr()
+    ptr = b2t.getref_base()._obj.container._as_ptr()
     assert lltype.typeOf(ptr) == lltype.Ptr(LLtypeMixin.S)
     assert ptr.a == 111
     assert ptr.b == lltype.nullptr(LLtypeMixin.NODE)
 
 
 def test_virtual_adder_pending_fields():
-    b2s, b4s = [BoxPtr(), BoxPtr()]
+    b2s, b4s = [InputArgRef(), InputArgRef()]
     storage = Storage()
     memo = ResumeDataLoopMemo(FakeMetaInterpStaticData())
-    modifier = ResumeDataVirtualAdder(storage, storage, memo)
+    modifier = ResumeDataVirtualAdder(None, storage, storage, memo)
     modifier.liveboxes_from_env = {}
     modifier.liveboxes = {}
     modifier.vfieldboxes = {}
 
-    v2 = OptValue(b2s)
-    v4 = OptValue(b4s)
     modifier.register_box(b2s)
     modifier.register_box(b4s)
 
-    values = {b4s: v4, b2s: v2}
     liveboxes = []
-    modifier._number_virtuals(liveboxes, FakeOptimizer(values), 0)
+    modifier._number_virtuals(liveboxes, FakeOptimizer(), 0)
     assert liveboxes == [b2s, b4s] or liveboxes == [b4s, b2s]
-    modifier._add_pending_fields([(LLtypeMixin.nextdescr, b2s, b4s, -1)])
+    modifier._add_pending_fields(FakeOptimizer(), [
+        ResOperation(rop.SETFIELD_GC, [b2s, b4s], descr=LLtypeMixin.nextdescr)])
     storage.rd_consts = memo.consts[:]
     storage.rd_numb = None
     # resume
     demo55.next = lltype.nullptr(LLtypeMixin.NODE)
-    b2t = BoxPtr(demo55o)
-    b4t = BoxPtr(demo66o)
+    b2t = InputArgRef(demo55o)
+    b4t = InputArgRef(demo66o)
     newboxes = _resume_remap(liveboxes, [b2s, b4s], b2t, b4t)
 
     metainterp = MyMetaInterp()
@@ -1403,18 +1412,23 @@ def test_virtual_adder_pending_fields_and_arrayitems():
     class Storage(object):
         pass
     storage = Storage()
-    modifier = ResumeDataVirtualAdder(storage, storage, None)
-    modifier._add_pending_fields([])
+    modifier = ResumeDataVirtualAdder(None, storage, storage, None)
+    modifier._add_pending_fields(None, [])
     assert not storage.rd_pendingfields
     #
-    class FieldDescr(object):
-        pass
+    class FieldDescr(AbstractDescr):
+        def is_array_of_primitives(self):
+            return False
     field_a = FieldDescr()
     storage = Storage()
-    modifier = ResumeDataVirtualAdder(storage, storage, None)
-    modifier.liveboxes_from_env = {42: rffi.cast(rffi.SHORT, 1042),
-                                   61: rffi.cast(rffi.SHORT, 1061)}
-    modifier._add_pending_fields([(field_a, 42, 61, -1)])
+    modifier = ResumeDataVirtualAdder(None, storage, storage, None)
+    a = InputArgInt()
+    b = InputArgInt()
+    modifier.liveboxes_from_env = {a: rffi.cast(rffi.SHORT, 1042),
+                                   b: rffi.cast(rffi.SHORT, 1061)}
+    modifier._add_pending_fields(FakeOptimizer(), [
+        ResOperation(rop.SETFIELD_GC, [a, b],
+                     descr=field_a)])
     pf = storage.rd_pendingfields
     assert len(pf) == 1
     assert (annlowlevel.cast_base_ptr_to_instance(FieldDescr, pf[0].lldescr)
@@ -1425,13 +1439,20 @@ def test_virtual_adder_pending_fields_and_arrayitems():
     #
     array_a = FieldDescr()
     storage = Storage()
-    modifier = ResumeDataVirtualAdder(storage, storage, None)
-    modifier.liveboxes_from_env = {42: rffi.cast(rffi.SHORT, 1042),
-                                   61: rffi.cast(rffi.SHORT, 1061),
-                                   62: rffi.cast(rffi.SHORT, 1062),
-                                   63: rffi.cast(rffi.SHORT, 1063)}
-    modifier._add_pending_fields([(array_a, 42, 61, 0),
-                                  (array_a, 42, 62, 2147483647)])
+    modifier = ResumeDataVirtualAdder(None, storage, storage, None)
+    a42 = InputArgInt()
+    a61 = InputArgInt()
+    a62 = InputArgInt()
+    a63 = InputArgInt()
+    modifier.liveboxes_from_env = {a42: rffi.cast(rffi.SHORT, 1042),
+                                   a61: rffi.cast(rffi.SHORT, 1061),
+                                   a62: rffi.cast(rffi.SHORT, 1062),
+                                   a63: rffi.cast(rffi.SHORT, 1063)}
+    modifier._add_pending_fields(FakeOptimizer(), [
+        ResOperation(rop.SETARRAYITEM_GC, [a42, ConstInt(0), a61],
+                     descr=array_a),
+        ResOperation(rop.SETARRAYITEM_GC, [a42, ConstInt(2147483647), a62],
+                     descr=array_a)])
     pf = storage.rd_pendingfields
     assert len(pf) == 2
     assert (annlowlevel.cast_base_ptr_to_instance(FieldDescr, pf[0].lldescr)
@@ -1445,8 +1466,12 @@ def test_virtual_adder_pending_fields_and_arrayitems():
     assert rffi.cast(lltype.Signed, pf[1].fieldnum) == 1062
     assert rffi.cast(lltype.Signed, pf[1].itemindex) == 2147483647
     #
-    py.test.raises(TagOverflow, modifier._add_pending_fields,
-                   [(array_a, 42, 63, 2147483648)])
+    if sys.maxint >= 2147483648:
+        py.test.raises(TagOverflow, modifier._add_pending_fields,
+                       FakeOptimizer(),
+                       [ResOperation(rop.SETARRAYITEM_GC,
+                                     [a42, ConstInt(2147483648), a63],
+                                     descr=array_a)])
 
 def test_resume_reader_fields_and_arrayitems():
     class ResumeReader(AbstractResumeDataReader):
@@ -1506,7 +1531,7 @@ def test_invalidation_needed():
     metainterp_sd = FakeMetaInterpStaticData()
     metainterp_sd.options = options
     memo = ResumeDataLoopMemo(metainterp_sd)
-    modifier = ResumeDataVirtualAdder(None, None, memo)
+    modifier = ResumeDataVirtualAdder(None, None, None, memo)
 
     for i in range(5):
         assert not modifier._invalidation_needed(5, i)
