@@ -2,7 +2,6 @@ import sys
 import types
 
 from rpython.flowspace.model import Constant
-from rpython.flowspace.operation import op
 from rpython.annotator import description, model as annmodel
 from rpython.rlib.objectmodel import UnboxedValue
 from rpython.tool.pairtype import pairtype, pair
@@ -97,8 +96,7 @@ def buildinstancerepr(rtyper, classdef, gcflavor='gc'):
         unboxed = [subdef for subdef in classdef.getallsubdefs() if
             subdef.classdesc.pyobj is not None and
             issubclass(subdef.classdesc.pyobj, UnboxedValue)]
-        virtualizable = classdef.classdesc.read_attribute(
-            '_virtualizable_', Constant(False)).value
+        virtualizable = classdef.classdesc.get_param('_virtualizable_', False)
     config = rtyper.annotator.translator.config
     usetagging = len(unboxed) != 0 and config.translation.taggedpointers
 
@@ -529,26 +527,24 @@ class InstanceRepr(Repr):
         self.allinstancefields = allinstancefields
 
     def _check_for_immutable_hints(self, hints):
-        loc = self.classdef.classdesc.lookup('_immutable_')
-        if loc is not None:
-            if loc is not self.classdef.classdesc:
+        hints = hints.copy()
+        classdesc = self.classdef.classdesc
+        immut = classdesc.get_param('_immutable_', inherit=False)
+        if immut is None:
+            if classdesc.get_param('_immutable_', inherit=True):
                 raise ImmutableConflictError(
                     "class %r inherits from its parent _immutable_=True, "
                     "so it should also declare _immutable_=True" % (
                         self.classdef,))
-            if loc.classdict.get('_immutable_').value is not True:
-                raise TyperError(
-                    "class %r: _immutable_ = something else than True" % (
-                        self.classdef,))
-            hints = hints.copy()
+        elif immut is not True:
+            raise TyperError(
+                "class %r: _immutable_ = something else than True" % (
+                    self.classdef,))
+        else:
             hints['immutable'] = True
-        self.immutable_field_set = set()  # unless overwritten below
-        if self.classdef.classdesc.lookup('_immutable_fields_') is not None:
-            hints = hints.copy()
-            immutable_fields = self.classdef.classdesc.classdict.get(
-                '_immutable_fields_')
-            if immutable_fields is not None:
-                self.immutable_field_set = set(immutable_fields.value)
+        self.immutable_field_set = classdesc.immutable_fields
+        if (classdesc.immutable_fields or
+                'immutable_fields' in self.rbase.object_type._hints):
             accessor = FieldListAccessor()
             hints['immutable_fields'] = accessor
         return hints
