@@ -25,22 +25,12 @@ def teardown_module(mod):
     py.log._setstate(mod.logstate)
 
 
-
-def timelog(prefix, call, *args, **kwds):
-    #import time
-    #print prefix, "...",
-    #start = time.time()
-    res = call(*args, **kwds)
-    #elapsed = time.time() - start
-    #print "%.2f secs" % (elapsed,)
-    return res
-
 def gengraph(func, argtypes=[], viewbefore='auto', policy=None,
              backendopt=False, config=None, **extraconfigopts):
     t = TranslationContext(config=config)
     t.config.set(**extraconfigopts)
     a = t.buildannotator(policy=policy)
-    timelog("annotating", a.build_types, func, argtypes, main_entry_point=True)
+    a.build_types(func, argtypes, main_entry_point=True)
     a.validate()
     if viewbefore == 'auto':
         viewbefore = getattr(option, 'view', False)
@@ -49,13 +39,13 @@ def gengraph(func, argtypes=[], viewbefore='auto', policy=None,
         t.view()
     global typer # we need it for find_exception
     typer = t.buildrtyper()
-    timelog("rtyper-specializing", typer.specialize)
+    typer.specialize()
     #t.view()
-    timelog("checking graphs", t.checkgraphs)
+    t.checkgraphs()
     if backendopt:
         from rpython.translator.backendopt.all import backend_optimizations
         backend_optimizations(t)
-        timelog("checking graphs", t.checkgraphs)
+        t.checkgraphs()
         if viewbefore:
             t.view()
     desc = t.annotator.bookkeeper.getdesc(func)
@@ -69,8 +59,7 @@ def clear_tcache():
     _tcache.clear()
 
 def get_interpreter(func, values, view='auto', viewbefore='auto', policy=None,
-                    type_system="lltype", backendopt=False, config=None,
-                    **extraconfigopts):
+                    backendopt=False, config=None, **extraconfigopts):
     extra_key = [(key, value) for key, value in extraconfigopts.iteritems()]
     extra_key.sort()
     extra_key = tuple(extra_key)
@@ -103,10 +92,8 @@ def get_interpreter(func, values, view='auto', viewbefore='auto', policy=None,
     return interp, graph
 
 def interpret(func, values, view='auto', viewbefore='auto', policy=None,
-              type_system="lltype", backendopt=False, config=None,
-              malloc_check=True, **kwargs):
+              backendopt=False, config=None, malloc_check=True, **kwargs):
     interp, graph = get_interpreter(func, values, view, viewbefore, policy,
-                                    type_system=type_system,
                                     backendopt=backendopt, config=config,
                                     **kwargs)
     if not malloc_check:
@@ -122,10 +109,9 @@ def interpret(func, values, view='auto', viewbefore='auto', policy=None,
     return result
 
 def interpret_raises(exc, func, values, view='auto', viewbefore='auto',
-                     policy=None, type_system="lltype",
+                     policy=None,
                      backendopt=False):
     interp, graph  = get_interpreter(func, values, view, viewbefore, policy,
-                                     type_system=type_system,
                                      backendopt=backendopt)
     info = py.test.raises(LLException, "interp.eval_graph(graph, values)")
     try:
