@@ -531,6 +531,7 @@ class TestStandalone(StandaloneTests):
             py.test.skip("requires fork()")
 
         def entry_point(argv):
+            print "parentpid =", os.getpid()
             debug_start("foo")
             debug_print("test line")
             childpid = os.fork()
@@ -543,40 +544,46 @@ class TestStandalone(StandaloneTests):
         t, cbuilder = self.compile(entry_point)
         path = udir.join('test_debug_print_fork.log')
         out, err = cbuilder.cmdexec("", err=True,
-                                    env={'PYPYLOG': ':%s' % path})
+                                    env={'PYPYLOG': ':%s.%%d' % path})
         assert not err
+        import time
+        time.sleep(0.5)    # time for the forked children to finish
         #
-        f = open(str(path), 'r')
+        lines = out.splitlines()
+        assert lines[-1].startswith('parentpid = ')
+        parentpid = int(lines[-1][12:])
+        #
+        f = open('%s.%d' % (path, parentpid), 'r')
         lines = f.readlines()
         f.close()
         assert '{foo' in lines[0]
         assert lines[1] == "test line\n"
-        offset1 = len(lines[0]) + len(lines[1])
+        #offset1 = len(lines[0]) + len(lines[1])
         assert lines[2].startswith('childpid = ')
         childpid = int(lines[2][11:])
         assert childpid != 0
         assert 'foo}' in lines[3]
         assert len(lines) == 4
         #
-        f = open('%s.fork%d' % (path, childpid), 'r')
+        f = open('%s.%d' % (path, childpid), 'r')
         lines = f.readlines()
         f.close()
-        assert lines[0] == 'FORKED: %d %s\n' % (offset1, path)
-        assert lines[1] == 'childpid = 0\n'
-        offset2 = len(lines[0]) + len(lines[1])
-        assert lines[2].startswith('childpid2 = ')
-        childpid2 = int(lines[2][11:])
+        #assert lines[0] == 'FORKED: %d %s\n' % (offset1, path)
+        assert lines[0] == 'childpid = 0\n'
+        #offset2 = len(lines[0]) + len(lines[1])
+        assert lines[1].startswith('childpid2 = ')
+        childpid2 = int(lines[1][11:])
         assert childpid2 != 0
-        assert 'foo}' in lines[3]
-        assert len(lines) == 4
-        #
-        f = open('%s.fork%d' % (path, childpid2), 'r')
-        lines = f.readlines()
-        f.close()
-        assert lines[0] == 'FORKED: %d %s.fork%d\n' % (offset2, path, childpid)
-        assert lines[1] == 'childpid2 = 0\n'
         assert 'foo}' in lines[2]
         assert len(lines) == 3
+        #
+        f = open('%s.%d' % (path, childpid2), 'r')
+        lines = f.readlines()
+        f.close()
+        #assert lines[0] == 'FORKED: %d %s.fork%d\n' % (offset2, path, childpid)
+        assert lines[0] == 'childpid2 = 0\n'
+        assert 'foo}' in lines[1]
+        assert len(lines) == 2
 
     def test_debug_flush_at_exit(self):
         def entry_point(argv):
